@@ -24,7 +24,7 @@
 from __future__ import unicode_literals
 import sys, os, re, codecs, json, argparse, getpass, base64
 # import class and constants
-from datetime import datetime
+from datetime import datetime, timedelta
 try:
     from urllib.parse import quote_plus
 except ImportError:
@@ -112,7 +112,11 @@ attr_translations = {'sAMAccountName':'SAM Name',
                      'lockoutThreshold':'Lockout Threshold',
                      'maxPwdAge':'Max password age',
                      'minPwdAge':'Min password age',
-                     'minPwdLength':'Min password length'}
+                     'minPwdLength':'Min password length',
+                     'pwdHistoryLength':'Password history length',
+                     'pwdProperties':'Password properties',
+                     'ms-DS-MachineAccountQuota':'Machine Account Quota',
+                     'flatName':'NETBIOS Domain name'}
 
 MINIMAL_COMPUTERATTRIBUTES = ['cn', 'sAMAccountName', 'dNSHostName', 'operatingSystem', 'operatingSystemServicePack', 'operatingSystemVersion', 'lastLogon', 'userAccountControl', 'whenCreated', 'objectSid', 'description', 'objectClass']
 MINIMAL_USERATTRIBUTES = ['cn', 'name', 'sAMAccountName', 'memberOf', 'primaryGroupId', 'whenCreated', 'whenChanged', 'lastLogon', 'userAccountControl', 'pwdLastSet', 'objectSid', 'description', 'objectClass']
@@ -378,7 +382,11 @@ class domainDumper(object):
             except (LDAPAttributeError, LDAPCursorError):
                 ugroups = []
             #Add the user default group
-            ugroups.append(self.getGroupCnFromDn(self.groups_dnmap[user.primaryGroupId.value]))
+            try:
+                ugroups.append(self.getGroupCnFromDn(self.groups_dnmap[user.primaryGroupId.value]))
+            # Sometimes we can't query this group or it doesn't exist
+            except KeyError:
+                pass
             for group in ugroups:
                 try:
                     groupsdict[group].append(user)
@@ -431,7 +439,7 @@ class reportWriter(object):
         #In grouped view, don't include the memberOf property to reduce output size
         self.userattributes_grouped = ['cn', 'name', 'sAMAccountName', 'whenCreated', 'whenChanged', 'lastLogon', 'userAccountControl', 'pwdLastSet', 'objectSid', 'description']
         self.groupattributes = ['cn', 'sAMAccountName', 'memberOf', 'description', 'whenCreated', 'whenChanged', 'objectSid']
-        self.policyattributes = ['cn', 'lockOutObservationWindow', 'lockoutDuration', 'lockoutThreshold', 'maxPwdAge', 'minPwdAge', 'minPwdLength', 'pwdHistoryLength', 'pwdProperties']
+        self.policyattributes = ['distinguishedName', 'lockOutObservationWindow', 'lockoutDuration', 'lockoutThreshold', 'maxPwdAge', 'minPwdAge', 'minPwdLength', 'pwdHistoryLength', 'pwdProperties', 'ms-DS-MachineAccountQuota']
         self.trustattributes = ['cn', 'flatName', 'securityIdentifier', 'trustAttributes', 'trustDirection', 'trustType']
 
     #Escape HTML special chars
@@ -446,15 +454,23 @@ class reportWriter(object):
 
     #Convert password max age (in 100 nanoseconds), to days
     def nsToDays(self, length):
-        return abs(length) * .0000001 / 86400
+        # ldap3 >= 2.6 returns timedelta
+        if isinstance(length, timedelta):
+            return length.total_seconds() / 86400
+        else:
+            return abs(length) * .0000001 / 86400
 
     def nsToMinutes(self, length):
-        return abs(length) * .0000001 / 60
+        # ldap3 >= 2.6 returns timedelta
+        if isinstance(length, timedelta):
+            return length.total_seconds() / 60
+        else:
+            return abs(length) * .0000001 / 60
 
     #Parse bitwise flags into a list
     def parseFlags(self, attr, flags_def):
         outflags = []
-        if attr is None:
+        if attr is None or attr.value is None:
             return outflags
         for flag, val in iteritems(flags_def):
             if attr.value & val:
@@ -594,7 +610,10 @@ class reportWriter(object):
             return self.formatGroupsHtml(att.values)
         #Primary group
         if aname == 'primarygroupid':
-            return self.formatGroupsHtml([self.dd.groups_dnmap[att.value]])
+            try:
+                return self.formatGroupsHtml([self.dd.groups_dnmap[att.value]])
+            except KeyError:
+                return 'NOT FOUND!'
         #Pwd flags
         if aname == 'pwdproperties':
             return ', '.join(self.parseFlags(att, pwd_flags))
@@ -677,7 +696,10 @@ class reportWriter(object):
         if aname == 'member' or aname == 'memberof' and type(att.values) is list:
             return self.formatGroupsGrep(att.values)
         if aname == 'primarygroupid':
-            return self.formatGroupsGrep([self.dd.groups_dnmap[att.value]])
+            try:
+                return self.formatGroupsGrep([self.dd.groups_dnmap[att.value]])
+            except KeyError:
+                return 'NOT FOUND!'
         #Domain trust flags
         if aname == 'trustattributes':
             return ', '.join(self.parseFlags(att, trust_flags))
